/**
 * jQuery template interface
 *
 */

(function($) {
	$.fn.serializeObject = function () {
    var o = {};
    var a = this.serializeArray();
    $.each(a, function() {
        if (o[this.name] !== undefined) {
            if (!o[this.name].push) {
                o[this.name] = [o[this.name]];
            }
            o[this.name].push(this.value || '');
        } else {
            o[this.name] = this.value || '';
        }
    });
    return o;
	};
})(jQuery);

(function($) {
	Handlebars._compiled_templates_ = {};

	/**
	 * apply template to selected DOM element
	 * <name> the selector of the template script
	 * <data> JSON object
	 * <options> - optional
	 *
	 * e.g.
	 * $("#my-placeholder").template ("#my-template", {'foo':'bar'});
	 *
	 */

	$.fn.template = function (name, data, options) {
		if (!name) throw "no template name";

		var dom = $(name);
		var tmp = dom.html();

		if (!Handlebars._compiled_templates_[tmp]) {
			Handlebars._compiled_templates_[tmp] = Handlebars.compile(tmp);
		}

		var done = options ? options.placeTemplate : null;
		var after = options ? options.applyTemplate : null;
		var before = options ? options.prepareData : null;

		if (before) {
			data = before.call(options, data);
		}

		var text = Handlebars._compiled_templates_[tmp](data);
		var aa = this.get(0);

		if (aa.children.length == 0) {
			this.html(text);
			this.contents().attr("data-template", dom.get(0).id);

			if (after) after.call(options, aa);
		}
		else {
			var tt = $("#_temp_dom_generator");
			if (tt.length == 0) tt = $('<div id="_temp_dom_generator" style="display:none;"></div>').appendTo("body");
			tt.html(text);
			tt.contents().attr("data-template", dom.get(0).id);
			var bb = tt.get(0);
			if (bb.children.length > 1) throw "children > 1";
			if (aa.children.length > 1) throw "children > 1";

			if (after) after.call(options, bb.children[0]);

			aa.replaceChild(bb.children[0], aa.children[0]);
		}

		if (done) done.call(options);
		return this.contents();
	};

	$.fn.relatedData = function(name) {
		var data = this.data(name);
		if (data) return data;
		else return this.parents("[data-" + name + "]").data(name);
	};
})(jQuery);


/**
 * JCR/REST interface
 *
 */

(function($) {

	/**
	 * (un)mangle the jcr names e.g. _jcr_title -> jcr:title
	 *
	 */

	$.__unfixName = function(name) {
		var v = name.replace(/^_(\w+)_(\w+)$/, '$1:$2');
		return v;
	};

	$.__unfixData = function(data) {
		for (var key in data) {
			if (!data.hasOwnProperty(key)) continue;

			var val = data[key];
			if (key.indexOf ('_') == 0) {
				if (key.lastIndexOf ('_') == key.length-1) delete data[key];
				else {
					var k = key.replace(/^_(\w+)_(\w+)$/, '$1:$2');
					data[k] = val;
					delete data[key];
				}
			}
		}
		return data;
	};


	/**
	 * mangle the jcr names and converts the data to array (if necessary)
	 *
	 */

	$.__fixDataAsArray = function(data, path, filter) {
		var rv = [];

		for (var key in data) {
			if (!data.hasOwnProperty(key)) continue;

			var val = data[key];

			if (filter && filter(val) == false) continue;
			if (key.match(/:/)) {
				key = "_" + key.replace(/:/g, "_");
			}

			if (typeof val == 'string' || typeof val == 'number' || typeof val == 'boolean') {}
			else {
				$.__fixData(val, path);
				val['_key_'] = key;

				rv.push(val);
			}
		}

		return rv;
	};


	/**
	 * mangle the jcr names
	 *
	 */

	$.__fixData = function(data, path, filter) {
		if (path) data["_jcr_path_"] = path;
		for (var key in data) {
			if (!data.hasOwnProperty(key)) continue;

			var val = data[key];
			if (filter && filter(val) == false) continue;

			if (key.match(/:/)) {
				var nkey = "_" + key.replace(/:/g, "_");
				data[nkey] = val;
				delete data[key];
				key = nkey;
			}

			if (typeof val == 'string' || typeof val == 'number' || typeof val == 'boolean') {}
			else {
				$.__fixData(val, path + "/" + key);
			}
		}
		return data;
	};

/************************************************************************
 ** @author oflorian
 ** @since Sat Jun 23 10:44:03 2012
 ************************************************************************/

	$.fn.submitNodes = function(func) {
		var $f = this;
		var f  = this.get (0);
		if (f.tagName != "FORM") {
			$f = this.parents ("FORM");
			f  = $f.get (0);
		}
		
		var data = $f.serialize();
		var path = f.action;
		if (func) {
			var rv = $.ajax({
				url: path,
				type: 'POST',
				data: data,
				asunc: true,
				dataType: 'json',
				success: function(data) {
					func(data);
				}
			});
			return rv;
		}
		else {
			var rv = $.ajax({
				url: path,
				type: 'POST',
				data: data,
				async: false,
				dataType: 'json'
			});
			return rv;
		}

		return true;
	};

	$.fn.submitToModel = function(model) {
		var $f = this;
		var f  = this.get (0);
		if (f.tagName != "FORM") {
			$f = this.parents ("FORM");
			f  = $f.get (0);
		}
		
		var vals = {};

		var data = $("input", $f).each (function (n, it) {
			var n = it.name;
			var v = it.value;
			if (n) {
				vals[n] = v;
			}
		});
		model.set (vals);
		model.save ();
		return true;
	};


/************************************************************************
 ** @author oflorian
 ** @since Sat Jun 23 10:43:56 2012
 ************************************************************************/

	$.saveNodes = function(path, data, func) {
		if (func) {
			var rv = $.ajax({
				url: path,
				type: 'POST',
				data: data,
				asunc: true,
				dataType: 'json',
				success: function(data) {
					func(data);
				}
			});
			return rv;
		}
		else {
			var rv = $.ajax({
				url: path,
				type: 'POST',
				data: data,
				async: false,
				dataType: 'json'
			});
			return rv;
		}
	};

/************************************************************************
 ** @author oflorian
 ** @since Sat Jun 23 10:43:48 2012
 ************************************************************************/

	$.loadNodes = function(path, func) {
		if (func) {
			var rv = $.get(path, function(data) {
				$.__fixData(path, data);
				func(data);
			});
			return rv;
		}
		else {
			var rv = $.ajax({
				url: path,
				async: false,
				dataType: 'json'
			});
			var data = $.parseJSON(rv.responseText);
			$.__fixData(data, path);
			return data;
		}
	};

/************************************************************************
 ** @author oflorian
 ** @since Sat Jun 23 10:43:52 2012
 ************************************************************************/

	$.moveAndOrderNode = function(from, to, pos, func) {
		//var form = [{name:':operation', value:'move'},{name:':applyTo', value:from},{name:':dest', value:to},{name:':order', value:pos}];
		var form = [{
			name: ':operation',
			value: 'move'
		}, {
			name: ':dest',
			value: to
		}, {
			name: ':order',
			value: pos
		}];
		$.post(from, form, function(response) {
			if (func) func();
		});
	};

/************************************************************************
 ** @author oflorian
 ** @since Sat Jun 23 10:44:30 2012
 ************************************************************************/

	$.moveNode = function(from, to, func) {
		var form = [{
			name: ':operation',
			value: 'move'
		}, {
			name: ':applyTo',
			value: from
		}, {
			name: ':dest',
			value: to
		}];
		$.post('/content/outliner', form, function(response) {
			if (func) {
				func();
			}
		});
	};

/************************************************************************
 ** @author oflorian
 ** @since Sat Jun 23 10:44:36 2012
 ************************************************************************/

	$.deleteNode = function(path, func) {
		var node = nodeBaseName(path);
		var dir = nodeDirName(path);
		var form = [{
			name: ':operation',
			value: 'delete'
		}, {
			name: ':applyTo',
			value: node
		}];
		$.post(dir, form, function(response) {
			if (func) {
				func();
			}
		});
	};
})(jQuery);
